/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "CollectionView/collectionscene.h"

#include "CollectionModel/collectionmodel.h"
#include "MediaItem/mediaitem.h"
#include "Database/database.h"

#include "debug.h"

#include <QSqlQuery>


/*******************************************************************************
    CollectionScene::rateTrack
*******************************************************************************/
void CollectionScene::rateTrack(QGraphicsItem* gi)
{
    TrackGraphicItem *item = static_cast<TrackGraphicItem*>(gi);

    MEDIA::TrackPtr track = item->media;
    if(track->id != -1) {
      Database db;
      if (!db.connect()) return;

      QSqlQuery q("", *db.sqlDb());
      q.prepare("UPDATE `tracks` SET `rating`=:rat WHERE `id`=:id;");
      q.bindValue(":rat", track->rating );
      q.bindValue(":id", track->id );
      q.exec();

      MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(track->parent());
      if(!album->isUserRating)
         album->rating = _collectionModel->getItemAutoRating(album);

      MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(album->parent());
      if(!artist->isUserRating)
         artist->rating = _collectionModel->getItemAutoRating(artist);
     }
}

/*******************************************************************************
    CollectionScene::rateArtist
*******************************************************************************/
void CollectionScene::rateArtist(QGraphicsItem* gi)
{
    ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(gi);

    MEDIA::ArtistPtr artist = item->media;

    if(artist->id != -1) {
      Database db;
      if (!db.connect()) return;

      QSqlQuery q("", *db.sqlDb());
      q.prepare("UPDATE `artists` SET `rating`=:rat WHERE `id`=:id;");
      q.bindValue(":rat", artist->rating );
      q.bindValue(":id", artist->id );
      q.exec();
     }
}

/*******************************************************************************
    CollectionScene::rateAlbum
*******************************************************************************/
void CollectionScene::rateAlbum(QGraphicsItem* gi)
{
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem *>(gi);

    MEDIA::AlbumPtr album = item->media;
    if(album->id != -1) {
      Database db;
      if (!db.connect()) return;

      QSqlQuery q("", *db.sqlDb());
      q.prepare("UPDATE `albums` SET `rating`=:rat WHERE `id`=:id;");
      q.bindValue(":rat", album->rating );
      q.bindValue(":id", album->id );
      q.exec();

      MEDIA::ArtistPtr artist = MEDIA::ArtistPtr::staticCast(album->parent());
      if(!artist->isUserRating)
         artist->rating = _collectionModel->getItemAutoRating(artist);
     }
}
/*******************************************************************************
    CollectionScene::rateSelection
*******************************************************************************/
void CollectionScene::rateSelection(QList<QGraphicsItem*> selection)
{
    Debug::debug() << "CollectionScene::rateSelection";
    QtConcurrent::run(this, &CollectionScene::threadedRateSelection, selection);
}

void CollectionScene::threadedRateSelection(QList<QGraphicsItem*> selection)
{
    foreach(QGraphicsItem* gi, selection)
    {
      switch(gi->type()) {
        case GraphicsItem::AlbumType      :
        case GraphicsItem::AlbumGenreType : rateAlbum(gi);break;
        case GraphicsItem::ArtistType     : rateArtist(gi);break;
        case GraphicsItem::TrackType      : rateTrack(gi);break;
        default: return;break;
      }
    } // end foreach
}



